package edu.jhu.cs.oose.group3.intelligentbook.junit;

import org.junit.*;

import edu.jhu.cs.oose.group3.intelligentbook.model.AddressContactItem;
import edu.jhu.cs.oose.group3.intelligentbook.model.Contact;
import edu.jhu.cs.oose.group3.intelligentbook.model.ContactGroup;
import edu.jhu.cs.oose.group3.intelligentbook.model.ContactItem;
import edu.jhu.cs.oose.group3.intelligentbook.model.EmailContactItem;
import edu.jhu.cs.oose.group3.intelligentbook.model.EnumeratedGroup;
import edu.jhu.cs.oose.group3.intelligentbook.model.EverybodyGroup;
import edu.jhu.cs.oose.group3.intelligentbook.model.PhoneContactItem;
import edu.jhu.cs.oose.group3.intelligentbook.util.ContactFinder;
import edu.jhu.cs.oose.group3.intelligentbook.util.Query;

public class ContactTest {
	Contact c = new Contact();

	@Test
	public void addContactItem() {
		// remove contact item if already existing
		if (c.getContactItem("Home") != null)
			c.removeContactItem("Home");
		Assert.assertTrue(c.getContactItem("Home") == null);

		c.addContactItem("Home", new AddressContactItem());
		Assert.assertTrue(c.getContactItem("Home") != null);
	}

	@Test
	public void addContactItemVisibilitiesFromStrings() {
		ContactGroup g = new EverybodyGroup();
		if (c.getGroup("Everybody") == null) {
			c.addGroup(g);
		}

		g = new EnumeratedGroup();
		g.setGroupName("Classmates");
		if (c.getGroup("Classmates") == null) {
			c.addGroup(g);
		}

		ContactGroup everybody = c.getGroup("Everybody");
		ContactGroup classmates = c.getGroup("Classmates");

		ContactItem ci = new AddressContactItem();
		if (c.getContactItem("Home") == null)
			c.addContactItem("Home", ci);

		String[] v1 = { "Everybody", "Classmates" };
		c.addContactItemVisibilitiesFromStrings(ci, v1);
		Assert.assertTrue(c.getContactItem("Home").isVisibleTo(everybody));
		Assert.assertTrue(c.getContactItem("Home").isVisibleTo(classmates));

		String[] v2 = { "Everybody" };
		c.addContactItemVisibilitiesFromStrings(ci, v2);
		Assert.assertTrue(c.getContactItem("Home").isVisibleTo(everybody));
		Assert.assertTrue(!c.getContactItem("Home").isVisibleTo(classmates));
	}

	@Test
	public void addGroup() {
		if (c.getGroup("Classmates") != null) {
			c.removeGroup(c.getGroup("Classmates"));
		}
		Assert.assertTrue(c.getGroup("Classmates") == null);

		ContactGroup g = new EnumeratedGroup();
		g.setGroupName("Classmates");
		c.addGroup(g);

		Assert.assertTrue(c.getGroup("Classmates") != null);
	}

	// @Test
	// public void findContacts() {
	// Query q = null;
	// ContactFinder finder = null;
	// c.findContacts(finder, q);
	//		
	// Assert.assertTrue(1 + 1 != 3);
	// }

	@Test
	public void getAllContactItems() {
		if (c.getContactItem("Office Phone") != null) {
			c.removeContactItem("Office Phone");
		}
		if (c.getContactItem("Personal Email") != null) {
			c.removeContactItem("Personal Email");
		}

		int initSize = c.getAllContactItems().size();

		c.addContactItem("Office Phone", new PhoneContactItem());
		c.addContactItem("Personal Email", new EmailContactItem());

		Assert.assertTrue(c.getAllContactItems().size() == initSize + 2);
	}

	@Test
	public void getAllGroups() {
		if (c.getGroup("Classmates") != null) {
			c.removeGroup(c.getGroup("Classmates"));
		}

		if (c.getGroup("Minions") != null) {
			c.removeGroup(c.getGroup("Minions"));
		}

		int initSize = c.getAllGroups().size();

		ContactGroup classmates = new EnumeratedGroup();
		classmates.setGroupName("Classmates");
		ContactGroup minions = new EnumeratedGroup();
		minions.setGroupName("Minions");

		c.addGroup(classmates);
		c.addGroup(minions);

		Assert.assertTrue(c.getAllGroups().size() == initSize + 2);
	}

	@Test
	public void getContactItem() {
		String name = "Home";
		ContactItem item = new AddressContactItem();
		item.setContents("16215 Alton Pkwy.");
		c.addContactItem(name, item);

		String contents = c.getContactItem("Home").getContents();
		Assert.assertTrue(contents == "16215 Alton Pkwy.");
	}

	@Test
	public void getGroup() {
		if (c.getGroup("Classmates") == null) {
			ContactGroup g = new EnumeratedGroup();
			g.setGroupName("Classmates");
			c.addGroup(g);
		}
		Assert.assertTrue(c.getGroup("Classmates") != null);
	}

	// @Test
	// public void getUserId() {
	// c.getUserId();
	// Assert.assertTrue(1 + 1 != 3);
	// }

	@Test
	public void removeContactItem() {
		if (c.getContactItem("Primary Phone") == null)
			c.addContactItem("Primary Phone", new PhoneContactItem());
		Assert.assertTrue(c.getContactItem("Primary Phone") != null);

		c.removeContactItem("Primary Phone");
		Assert.assertTrue(c.getContactItem("Primary Phone") == null);
	}

	@Test
	/** This method tests 
	 * 		public void removeGroup(ContactGroup g) */
	public void removeGroup() {
		ContactGroup g = new EnumeratedGroup();
		g.setGroupName("Classmates");
		if (c.getGroup("Classmates") == null) {
			c.addGroup(g);
		}
		Assert.assertTrue(c.getGroup("Classmates") != null);

		c.removeGroup(g);
		Assert.assertTrue(c.getGroup("Classmates") == null);
	}

	@Test
	/** This method tests 
	 * 		public void removeGroup(String name) */
	public void removeGroupByString() {
		ContactGroup g = new EnumeratedGroup();
		g.setGroupName("Classmates");
		if (c.getGroup("Classmates") == null) {
			c.addGroup(g);
		}
		Assert.assertTrue(c.getGroup("Classmates") != null);

		c.removeGroup("Classmates");
		Assert.assertTrue(c.getGroup("Classmates") == null);
	}

}
