package edu.jhu.cs.oose.group3.intelligentbook.model;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import edu.jhu.cs.oose.group3.intelligentbook.util.ContactFinder;
import edu.jhu.cs.oose.group3.intelligentbook.util.Query;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.MapKey;
import javax.persistence.OneToMany;


/** Represents a contact, each of which corresponds to a user */
@Entity
public class Contact {

	
	@Id
	@GeneratedValue
	private long userId;

	@OneToMany(fetch=FetchType.EAGER)
	private List<ContactGroup> contactGroups = new ArrayList<ContactGroup>();
	
	@OneToMany(fetch=FetchType.EAGER)
    @MapKey(name="name")
	@JoinTable(
			joinColumns = {@JoinColumn(name="userId")},
			inverseJoinColumns = {@JoinColumn(name="ContactItemId")}
	)
	private Map<String,ContactItem> contactItems = new HashMap<String,ContactItem>();
	
	/** Add a new group to this contact
	 * @param g
	 */
	public void addGroup(ContactGroup g) {
		contactGroups.add(g);
	}
	
	/** Remove a group from this contact.
	 * 
	 * Each group member will lose their association with the user unless
	 * they are also a member of a different group
	 * @param g
	 */
	public void removeGroup(ContactGroup g) {
		contactGroups.remove(g);
	}
	
	/** Remove a contact group by name */
	public void removeGroup(String name) {
		ContactGroup g = getGroup(name);
		if (g != null)
			removeGroup(g);		
	}
	
	/** Get the contact group associated with the given name */
	public ContactGroup getGroup(String name) {
		for (ContactGroup g : contactGroups) 
			if (g.getGroupName().equals(name))
				return g;
		return null;
	}
	
	/** Get a list of all of this contact's groups */
	public List<ContactGroup> getAllGroups() {
		return Collections.unmodifiableList(contactGroups);
	}
	
	public Map<String,ContactItem> getAllContactItems() {
		return Collections.unmodifiableMap(contactItems);
	}
	
	/** Associate a new contact item with this contact
	 * @param name
	 * @param item 
	 */
	public void addContactItem(String name, ContactItem item) {
		contactItems.put(name,item);
	}
	/** Remove a contact item from this contact
	 * @param name 
	 */
	public void removeContactItem(String name) {
		contactItems.remove(name);
	}
	
	/** Get a contact item by name */
	public ContactItem getContactItem(String name) {
		return contactItems.get(name);
	}
	
	/** Given an array of group names which can see a contact item, set visibilites */
	public void addContactItemVisibilitiesFromStrings(ContactItem ci, String[] visibilities) {
		ci.resetVisibility();
		for (String name : visibilities) {
			ci.setVisibleTo(getGroup(name), true);
		}
	}
	
	/** find a list of contacts using a specific ContactFinder and query
	 * @param finder 
	 * @param q */
	public List<Contact> findContacts(ContactFinder finder, Query q) {
		return null;
	}

	public long getUserId() {
		return userId;
	}

	private void addDefaultGroups() {
		contactGroups.add(new FriendGroup());
		contactGroups.add(new ColleagueGroup());
		contactGroups.add(new EverybodyGroup());
		contactGroups.add(new FriendOfFriendGroup());
	}
	
	public Contact() {
		addDefaultGroups();
	}

}
