package edu.jhu.cs.oose.group3.intelligentbook.model;

import java.util.Collections;
import java.util.Set;
import java.util.TreeSet;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.OneToMany;

/** A group whose members are enumerated and can be changed by the user */
@Entity
public class EnumeratedGroup extends ContactGroup {
	public String getName() {
		return name;
	}

	/** The user can rename a group */
	public void setName(String name) {
		this.name = name;
	}

	@OneToMany(fetch = FetchType.EAGER)
	@JoinTable(name = "group_members", joinColumns = { @JoinColumn(name = "groupId") }, inverseJoinColumns = { @JoinColumn(name = "userId") })
	private Set<Contact> members = new TreeSet<Contact>();

	/** Add a contact to the group */
	public void addContact(Contact c) {
		members.add(c);
	}

	/** Remove a contact */
	public void removeContact(Contact c) {
		members.remove(c);
	}

	@Override
	/**
	 * @param c the contact 
	 * @returns true when c is in this groups member set */
	public boolean hasContact(Contact c) {
		return members.contains(c);
	}

	@Override
	public Set<Contact> getAllContacts() {
		return Collections.unmodifiableSet(members);
	}
}
